@@ -1,11 +1,12 @@
 import os
 import signal
 from enum import Enum
+from itertools import count
 from typing import Any, Dict, List, MutableMapping, Optional
-from pydantic.main import BaseModel
 
 import toml
 from pydantic import BaseSettings, Field, validator
+from pydantic.main import BaseModel
 
 DEFAULT_IGNORES = [
     # Some of these are redundant
@@ -25,9 +26,24 @@ class Events(str, Enum):
     moved = 'moved'
 
 
+Increment = count(start=0)
+
+
+def NextIncrement():
+    global Increment
+    val = next(Increment)
+    return val
+
+
+def SetIncrement(start: int = 0):
+    global Increment
+    Increment = count(start=start)
+
+
 class ForemonConfig(BaseSettings):
 
-    alias:           str = Field('')
+    alias: str = Field('')
+    order: Optional[int]
 
     ################################
     # Script execution
@@ -69,10 +85,21 @@ class ForemonConfig(BaseSettings):
                 value = os.path.expandvars(value)
         return value
 
-    def get_env(self) -> MutableMapping[str, str]:
-        env = os.environ.copy()
-        env.update(self.environment)
-        return env
+    @validator('order')
+    def validate_order(cls, value) -> int:
+        """
+        If and `order` is set explicitly we increment the default anyway. This
+        will ensure default `order` values will have a consistant value related
+        to their delcaration in the config file.
+
+        Due to the order-of-operations used by Pydantic internally we can't use
+        the Field default_factory.
+        """
+
+        inc = NextIncrement()
+        if value is None:
+            value = inc
+        return value
 
     class Config:
         env_prefix = 'foremon_'
@@ -98,6 +125,23 @@ class ForemonConfig(BaseSettings):
             configs.append(obj)
         super().__init__(*args, **kwargs)
 
+    def get_env(self) -> MutableMapping[str, str]:
+        env = os.environ.copy()
+        env.update(self.environment)
+        return env
+
+    def get_configs(self) -> List['ForemonConfig']:
+        """
+        Generator yields all configs in order
+        """
+        configs = [self]
+        ordered: List[ForemonConfig] = []
+        while configs:
+            config = configs.pop(0)
+            configs.extend(config.configs)
+            ordered.append(config)
+        return sorted(ordered, key=lambda c: c.order)
+
 
 ForemonConfig.update_forward_refs()
 
@@ -119,6 +163,10 @@ class PyProjectConfig(BaseSettings):
 
     @classmethod
     def parse_toml(cls, text: str) -> 'PyProjectConfig':
+        # TODO - Do this without a global. We need to reset this to zero each
+        # time a config is reloaded to keep auto-values constant between config
+        # reloads.
+        SetIncrement(0)
         data = toml.loads(text)
         project = cls.parse_obj(data)
         return project
@@ -138,6 +186,8 @@ class ForemonOptions(BaseModel):
     use_all: bool = Field(False)
     verbose: bool = Field(False)
     auto_reload: bool = Field(True)
+    dwell: float = Field(0.1)
 
 
-__all__ = ['PyProjectConfig', 'ToolConfig', 'ForemonConfig', 'Events', 'ForemonOptions']
+__all__ = ['PyProjectConfig', 'ToolConfig',
+           'ForemonConfig', 'Events', 'ForemonOptions']
