@@ -1,5 +1,6 @@
 import asyncio
 import errno
+from foremon.debounce import Debounce
 import os.path as op
 from asyncio import BaseEventLoop, Queue
 from functools import partial
@@ -21,6 +22,7 @@ class Monitor:
 
     _loop: BaseEventLoop
     observer: Observer
+    debounce: Debounce
     pipe: Optional[TextIO]
     queue: Queue
     stop_timeout: int
@@ -31,13 +33,14 @@ class Monitor:
     is_terminating: bool
     is_paused: bool
 
-    def __init__(self, pipe=None, loop: BaseEventLoop = None):
+    def __init__(self, dwell: float = 0.1, pipe=None, loop: BaseEventLoop = None):
 
         if loop is None:
             loop = asyncio.get_event_loop()
 
         self.stop_timeout = 5
         self.observer = Observer()
+        self.debounce = Debounce(dwell, self.queue_task_event, loop=loop)
         self.pipe = pipe
         self._loop = loop
         self.queue = Queue()
@@ -78,7 +81,7 @@ class Monitor:
             raise ForemonError(
                 'no valid paths specified, cannot add watch task', errno.ENOENT)
 
-        callback = partial(self.queue_task_event, task)
+        callback = partial(self.debounce.submit_threadsafe, task)
 
         handler = PatternMatchingEventHandler(
             patterns=conf.patterns,
@@ -127,8 +130,11 @@ class Monitor:
         if self.is_terminating or self.is_paused:
             return
 
-        task = self.loop.create_task(self.run_task(task, ev))
-        self.loop.call_soon_threadsafe(self.queue.put_nowait, task)
+        if task.running:
+            task.terminate()
+
+        self.loop.call_soon_threadsafe(
+            self.queue.put_nowait, lambda: self.run_task(task, ev))
 
     async def run_task(self, task: ForemonTask, trigger: Any) -> None:
         if self.is_terminating or self.is_paused:
@@ -137,11 +143,7 @@ class Monitor:
         if not self.observer.is_alive():
             return
 
-        # bounce until task is done
-        if task in self.active_tasks:
-            return
-
-        if task.running:
+        if task.running or task in self.active_tasks:
             return
 
         self.active_tasks.add(task)
@@ -207,7 +209,13 @@ class Monitor:
         async def cradle():
             async for task in queueiter(self.queue):
                 try:
-                    await task
+                    while task:
+                        if asyncio.iscoroutinefunction(task):
+                            task = await task()
+                        elif asyncio.iscoroutine(task):
+                            task = await task
+                        elif callable(task):
+                            task = task()
                 except Exception as e:
                     display_error('fatal error, shutting down ...', e)
                     break
